# The software in this package is distributed under the GNU General
# Public License version 2 (with a special exception described below).
#
# A copy of GNU General Public License (GPL) is included in this distribution,
# in the file COPYING.GPL.
#
# As a special exception, if other files instantiate templates or use macros
# or inline functions from this file, or you compile this file and link it
# with other works to produce a work based on this file, this file
# does not by itself cause the resulting work to be covered
# by the GNU General Public License.
#
# However the source code for this file must still be made available
# in accordance with section (3) of the GNU General Public License.
#
# This exception does not invalidate any other reasons why a work based
# on this file might be covered by the GNU General Public License.

""" type_helper_unittest.py: unit tests for type_helpers

Tests classes and functions in type_helpers

Should be run from python2 and python3!

For help see :py:mod:`unittest`

.. codeauthor:: Intra2net
"""

import unittest

from type_helpers import is_unicode, isstr
from sys import version_info


is_py2 = version_info.major == 2

class TypeHelperTester(unittest.TestCase):

    def test_is_py2_or_py3(self):
        """ test that python version is 2 or 3

        when implementing type_helpers, there was no py4 and no idea what it
        might be like. Also will probably fail for python v1
        """
        self.assertIn(version_info.major, (2, 3))

    def test_is_unicode(self):
        """ tests function is_unicode """

        self.assertFalse(is_unicode(1))
        self.assertFalse(is_unicode([]))
        self.assertFalse(is_unicode(unittest.TestCase))
        self.assertFalse(is_unicode(type(unittest.TestCase)))

        if is_py2:
            self.assertTrue(is_unicode(u'bla'))
            self.assertTrue(eval("is_unicode(ur'bla')"))  # not allowed in py3!
            self.assertFalse(is_unicode('bla'))
            self.assertFalse(is_unicode(r'bla'))
        else:
            self.assertTrue(is_unicode('bla'))
            self.assertTrue(is_unicode(r'bla'))
            self.assertFalse(is_unicode(b'bla'))
            self.assertFalse(is_unicode(br'bla'))

    def test_isstr(self):
        """ test function isstr """

        tests = [
            ('abc', True), (u'abc', True), (r'abc', True),
            (1, False), (['a', 'b', 'c'], False), (('a', 'b', 'c'), False)]
        if not is_py2:
            tests.append((b'abc', False))  # b'' does not exist in py2

        for test_var, expected_result in tests:
            self.assertEqual(isstr(test_var), expected_result,
                             'isstr of var {0} (which is of type {1}) returned'
                             '{2} but expected {3}!'
                             .format(test_var, type(test_var), isstr(test_var),
                                     expected_result))


if __name__ == '__main__':
    unittest.main()
